/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.navigation.ReportNavigator");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.FolderWidget");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");

var FolderWidget					= com.ibm.rational.report.web.core.app.internal.common.FolderWidget;
var ReportClient 					= com.ibm.rational.report.web.core.client.internal.ReportClient;
var MessageFormatter				= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var ServiceResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var session							= com.ibm.team.repository.web.client.session;
var ServiceabilityMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");

/**
 * Displays the Reports drawer, report folders and reports on the left navigator.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.navigation.ReportNavigator",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/navigation/templates/ReportNavigator.html"),
	
	constructor: function(args) {
		this._controller = args.controller;
	},
	
	postCreate: function(){
		this._msgFormatter = new MessageFormatter(null);
	},
	
	/**
	 *  Synchronize the report tree in the web page with the updated tree fetched from the server.
	 *  If the report tree is not present in the web page it is created. If it is already present then 
	 *  only the necessary modifications take place (folder and report widgets are reused).  
	 */
	displayReportNodes: function() {
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var owner =  session.getAuthenticatedContributor();
		var params = {projectAreaId: projectArea.itemId, ownerId: owner.itemId, parentId: ""};
		
		var srh = new ServiceResponseHandler(this, "_getReportTreeNodesSuccess", "_remoteError");
		ReportClient.getReportTreeNodes(srh, params);		
	},
	
	_getReportTreeNodesSuccess: function(treeNodes) {
		var nodeMap = new Object();
		
		var domNodes = this.reportListDiv.childNodes;		
		for(var j=0;j<domNodes.length;j++){
			nodeMap[domNodes[j].id] = domNodes[j];
		}
		for (var i in treeNodes) {
			var treeNode = treeNodes[i];
			nodeMap[treeNode.itemId] = null;
			var widget = dijit.byId(treeNode.itemId);
			if(widget!=null && widget._parentWidget!=this.reportListDiv) {
				widget.destroyRecursive(false);
			} 
			var widget = dijit.byId(treeNode.itemId);
			if(widget==null) {
				var nodeItem = new FolderWidget(
					{ controller: this._controller,
						 id: treeNode.itemId, 
						 folder : treeNode, expand:false, 
						 parentWidget:this.reportListDiv
					});
				this.reportListDiv.appendChild(nodeItem.domNode);
			}
			else{
				widget._updateNode(treeNode);	
			}
		}
		for(var k=0;k<domNodes.length;k++){
			var domNode = nodeMap[domNodes[k].id];			
			if(domNode !=null){
				var widget = dijit.byId(domNodes[k].id);
				if(widget!=null) {
					widget.destroyRecursive(false);
					k--;
				}
			}			
		}
	},
	
	_remoteError: function(error) {
		var errorMsg = this._msgFormatter.formatErrorMessage(ServiceabilityMessages.INTERNAL_APPLICATION_ERROR, error);
		this._displayErrorMessage(errorMsg);
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	}

	
});

})();